.code

RGB2HSL proc uses esi edi ebx, dwRGB:DWORD
  ;--------------------------------------------------------------;
  ; By Greg Hoyer aka "Iblis"                                    ;
  ;                                                              ;
  ; RGB2HSL converts a COLORREF  oriented dword filled with 8bit ;
  ; Red/Green/Blue  values  (00ggbbrr) to  a similarly  oriented ;
  ; dword filled with Hue/Saturation/Luminance values (00llsshh) ;
  ; This procedure  returns the  full range,  from 0-255.   This ;
  ; offers slightly more  precision over Windows' "color picker" ;
  ; common dialog, which displays HSL values ranging from 0-240. ;
  ;                                                              ;
  ; It is important to note  that true HSL  values are  normally ;
  ; represented as  floating point  fractions from  0.0 to  1.0. ;
  ; As such, this  algorithm cannot  be used to  do the precise, ;
  ; consistent conversions  that may  be required  by heavy-duty ;
  ; graphics  applications.  To  get the  decimal  fraction  for ;
  ; the  returned values,  convert  the Hue,  Saturation, and/or ;
  ; Luminance values to floating  point, and then divide by 255. ;
  ;--------------------------------------------------------------;
        movzx         esi, byte ptr [dwRGB+2]
        movzx         edi, byte ptr [dwRGB+1]
        movzx         ebx, byte ptr [dwRGB+0]
        mov           cl, -1
        cmp           esi, edi
        ja            @cmp1
        xchg          esi, edi
        neg           cl
        shl           cl, 1
@cmp1:  cmp           edi, ebx
        jb            @cmp2
        xchg          edi, ebx
        neg           cl
@cmp2:  cmp           esi, ebx
        ja            @cmp3
        xchg          esi, ebx
        not           cl
@cmp3:  neg           ebx
        add           ebx, esi
        mov           eax, edi
        add           edi, esi
        jz            @done
        sub           esi, eax
        jz            @done
        mov           eax, esi
        shl           eax, 8
        sub           eax, esi
        push          edi
        cmp           edi, 0FFh
        jbe           @csat
        neg           edi
        add           edi, 510
@csat:  xor           edx, edx
        div           edi
        pop           edi
        shr           edi, 1
        shl           eax, 8
        or            edi, eax
        add           cl, 3
        jnc           @noneg
        neg           ebx
@noneg: shl           cl, 2
        mov           eax, 13135DB9h
        shr           eax, cl
        and           eax, 7
        mul           esi
        add           eax, ebx
        mov           ebx, eax
        shl           eax, 8
        sub           eax, ebx
        mov           ebx, esi
        shl           esi, 1
        lea           ebx, [ebx*4+esi]
        xor           edx, edx
        div           ebx
        shl           eax, 16
        or            eax, edi
@done:  bswap         eax
        shr           eax, 8
        ret

RGB2HSL endp



HSL2RGB proc uses esi edi ebx, dwHSL:DWORD
  ;--------------------------------------------------------------;
  ; By Greg Hoyer aka "Iblis"                                    ;
  ;                                                              ;
  ; HSL2RGB  does  the  opposite  of  RGB2HSL.   It  converts  a ;
  ; Hue/Saturation/Luminance  (00llsshh)  dword  back  into  its ;
  ; corresponding  RGB  COLORREF  (00bbggrr).  This  function is ;
  ; intented  to be  used exclusively  with RGB2HSL  (see above) ;
  ;                                                              ;
  ; If  you're  using  this for  your own  custom  color-chooser ;
  ; dialog, remember that the  values are in the range of 0-255. ;
  ; If you MUST emulate the Windows'  color-chooser, convert HSL ;
  ; values this way before you display them:                     ;
  ;                                                              ;
  ; display_value = ( x * 240 ) / 255                            ;
  ;                                                              ;
  ; ...where x represents any one of the HSL values.             ;
  ;--------------------------------------------------------------;

        movzx         ebx, byte ptr [dwHSL+2]
        lea           esi, [ebx*2]
        movzx         edi, byte ptr [dwHSL+1]
        xor           eax, eax
        mov           cl, 1
        cmp           bl, 7Fh
        ja            @lcase
        dec           al
        xor           ecx, ecx
@lcase: add           eax, edi
        mul           ebx
        or            ecx, ecx
        jz            @scase
        neg           eax
        mov           ecx, ebx
        add           ecx, edi
        mov           edx, ecx
        shl           ecx, 8
        sub           ecx, edx
        add           eax, ecx
@scase: xor           edx, edx
        xor           ecx, ecx
        dec           cl
        mov           edi, ecx
        div           ecx
        jz            @done
        mov           ecx, eax
        sub           esi, eax
        movzx         eax, byte ptr [dwHSL+0]
        mov           ebx, eax
        shl           eax, 1
        lea           eax, [ebx*4+eax]
        xor           edx, edx
        div           edi
        mov           ebx, eax
        mov           eax, ecx
        sub           eax, esi
        mul           edx
        push          ebx
        mov           ebx, ecx
        shl           ebx, 8
        sub           ebx, ecx
        sub           ebx, eax
        xchg          eax, ebx
        xor           edx, edx
        div           edi
        shl           eax, 24
        or            ecx, eax
        mov           eax, esi
        shl           eax, 8
        sub           eax, esi
        shl           esi, 16
        or            ecx, esi
        add           eax, ebx
        xor           edx, edx
        div           edi
        mov           ch, al
        mov           eax, ecx
        pop           ecx
        cmp           cl, 6
        jz            @done
        or            ecx, ecx
        jz            @done
        bswap         eax
        rol           eax, 8
        xchg          ah, al
        dec           ecx
        jz            @done
        ror           eax, 8
        xchg          ah, al
        dec           ecx
        jz            @done
        rol           eax, 8
        xchg          ah, al
        dec           ecx
        jz            @done
        bswap         eax
        rol           eax, 8
        xchg          ah, al
        dec           ecx
        jz            @done
        ror           eax, 8
        xchg          ah, al
@done:  and           eax, 0FFFFFFh
        ret

HSL2RGB endp
